/***** interface.c ************************************************
 * Description: Routine for gathering arguments from the command
 *              line.
 * Author: Bernhard Haubold, haubold@evolbio.mpg.de
 * Date: Sun Jun 20 13:12:10 2004.
 *****************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include "interface.h"
#include "eprintf.h"

Args *args;

Args *getArgs(int argc, char *argv[]){
  char c;
  char *optString = "hvt:s:i:";

  args = (Args *)emalloc(sizeof(Args));
  args->t = DEFAULT_T;
  args->i = 0;
  args->s = 0;
  args->i = 0;
  args->h = 0;
  args->v = 0;
  args->e = 0;

  c = getopt(argc, argv, optString);
  while(c != -1){
    switch(c){
    case 't':                           /* test type */
      args->t = optarg[0];
      if(args->t != 'm' && args->t != 's' && args->t != 'w'){
	printf("WARNING: %c is not one of the test types available; using Walsh's t-Test.\n", args->t);
	args->t = 'w';
      }
      break;
    case 'i':                           /* number of iterations for MC-test */
      args->i = atoi(optarg);
      break;
    case 's':                           /* seed for random number generator */
      args->s = atoi(optarg);
      break;
    case '?':                           /* fall-through is intentional */
    case 'h':                           /* print help */
      args->h = 1;
      break;
    case 'v':                           /* print version */
      args->v = 1;
      break;
    default:
      printf("# unknown argument: %c\n",c);
      args->e = 1;
      return args;
    }
    c = getopt(argc, argv, optString);
  }
  args->inputFiles = argv + optind;
  args->numInputFiles = argc - optind;
  if(!args->h && !args->v && args->numInputFiles < 2){
    printf("ERROR: Need at least two input files.\n");
    exit(-1);
  }
  if(args->t == 'm' && args->i == 0)
    args->i = DEFAULT_I;
  return args;
}


void printUsage(char *version){
  printf("Usage: %s [options] samples1.txt samples2.txt\n",progname());
  printf("Test the statistical significance of the difference between two means\n");
  printf("Example: testMeans -t m -i 1000 samples1.txt samples2.txt\n");
  printf("Data format: Name_1 Measurement_1 Measuremnet_2 ... Measurement_n\n");
  printf("             Name_n Measurement_1 Measuremnet_2 ... Measurement_n\n");
  printf("             ...\n");
  printf("             Name_m Measurement_1 Measuremnet_2 ... Measurement_n\n");
  printf("Options:\n");
  printf("\t[-t w|s|m test type (Welch|Student|Monte-Carlo); default: %c]\n", DEFAULT_T);
  printf("\t[-i <NUM> number of iterations for Monte-Carlo test; implies -t m; default: %d]\n",DEFAULT_I);
  printf("\t[-s <NUM> seed for random number generator; default: system, randomSeed.dat]\n");
  printf("\t[-h print this help message and exit]\n");
  printf("\t[-v print program information and exit]\n");
  exit(0);
}

void printSplash(char *version){
  printf("%s %s\n",progname(),version);
  printf("Written by Bernhard Haubold.\n");
  printf("Distributed under the GNU General Public License.\n");
  printf("Please send bug reports to haubold@evolbio.mpg.de\n");
  exit(0);
}
