/***** tTest.c ************************************
 * Description: 
 * Author: Bernhard Haubold, haubold@evolbio.mpg.de
 * Date: Thu Dec 15 11:45:46 2016
 **************************************************/
#include <stdio.h>
#include <math.h>
#include <float.h>
#include <gsl/gsl_statistics_double.h>
#include <gsl/gsl_cdf.h>
#include "testMeans.h"

Result *welch(Result *res, double *s1, int n1, double *s2, int n2){
  double m1, m2, v1, v2, t, nu, nu1, nu2, x;

  m1 = gsl_stats_mean(s1, 1, n1);
  res->m1 = m1;
  m2 = gsl_stats_mean(s2, 1, n2);
  res->m2 = m2;
  v1 = gsl_stats_variance(s1, 1, n1);
  res->v1 = v1;
  v2 = gsl_stats_variance(s2, 1, n2);
  res->v2 = v2;
  x = v1/n1 + v2/n2;
  if(x){
    t = (m1-m2) / sqrt(v1/n1 + v2/n2);
    res->t = t;
  }else{
    res->p = 1.0;
    res->t = -DBL_MAX;
    return res;
  }
  nu1 = n1 - 1;
  nu2 = n2 - 1;
  nu = pow((v1/n1 + v2/n2),2) / (v1*v1/n1/n1/nu1 + v2*v2/n2/n2/nu2);
  res->nu = nu;
  if(t > 0)
    res->p = gsl_cdf_tdist_Q(t, nu) * 2.;
  else
    res->p = gsl_cdf_tdist_P(t, nu) * 2.;
  if(res->p > 1.)
    res->p = 1.;
  return res;
}

Result *student(Result *res, double *s1, int n1, double *s2, int n2){
  double m1, m2, v1, v2, t, nu;

  m1 = gsl_stats_mean(s1, 1, n1);
  res->m1 = m1;
  m2 = gsl_stats_mean(s2, 1, n2);
  res->m2 = m2;
  v1 = gsl_stats_variance(s1, 1, n1);
  res->v1 = v1;
  v2 = gsl_stats_variance(s2, 1, n2);
  res->v2 = v2;
  if(!(v1 == 0. && v2 == 0.)){
    t = (m1-m2) / sqrt((n1-1)*v1 + (n2-1)*v2) * sqrt(n1*n2*(n1+n2-2)/(n1+n2));
    res->t = t;
  }else{
    res->p = 1.0;
    res->t = -DBL_MAX;
    return res;
  }
  nu = n1 + n2 - 2;
  res->nu = nu;
  if(t > 0)
    res->p = gsl_cdf_tdist_Q(t, nu) * 2.;
  else
    res->p = gsl_cdf_tdist_P(t, nu) * 2.;
  if(res->p > 1.)
    res->p = 1.;
  return res;
}
